/***********************************************************/
/******	    small TCC/C-Tutorial v0.2.0 - 16.04.2006  ******/
/***********************************************************

		Copyright (C) 2006 Martin Wendt

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.

		see "GNUGPL" for the GNUGPL
***********************************************************
  	   contact me under martin.wendt@gmx.de
***********************************************************
******  PLEASE READ 'enthusi1.txt' for the Tutorial  ******
***********************************************************

USE "tccrun enthusi1" to compile and type "enthusi1.run" to start it :)
Code well,
enthusi, Hamburg */

#include "/usr/lib/tcc/include/tcclib.h"
long double atof(const char *s);	/* declaration that is not in tcclib.h */

/* those would be needed for GCC 
	#include <stdlib.h>
	#include <stdio.h>    
*/
	#define CHECK printf("all still fine in line:   %d\n",__LINE__);
	#pragma STDC FP_CONTRACT ON

	struct pixel_t { unsigned char red;
			  	 unsigned char green;
				 unsigned char blue; } ;

int main(int argc, char *argv[])
	{
	/*
	all variables are complex.
	value_new = realpart + i*imgpart
	constant = realpart + i*imgpart

	do this iteration:
	value_new = value_old * value_old - constant
	value_new = value_old
	for ? times and check if value_new is still limited
	if it is not, plot pixel in a color depending on after
	how many iterations it became 'unlimited' big.
	*/

	
	FILE *dpic;	/* output image */
	int yp,xp;
	float ix,iy,ia,ib,nix,xs,ys;
	int loop;
	struct pixel_t pixel; 
	char filename[50];
	char command[50];

	/* use double ix,iy,ia,ib,nix,xs,ys;	*/
	/* and double steprange;				*/
	/* for higher maximum resolution		*/

	if (argc<7){
		argv[1]="-2.5";	/* lower border of the real-part 			*/
		argv[2]="-1.25";/* lower border of the img-part				*/
		argv[3]="2.5";	/* size of the region displayed				*/
		argv[4]="100";	/* max no. of iterations done per pixel		*/
		argv[5]="640";	/* default width							*/
		argv[6]="480";  /* default height							*/
		}; 				/* check enthusi1.txt for more info on this */

	const int width=atoi(argv[5]);	/* convert ascii (=text) to integer */
	const int height=atoi(argv[6]);
	const int maxloop=atoi(argv[4]);
	const float steprange=(float)(atof(argv[3])/height);
	ia=atof(argv[2]); /* convert ascii (=text) to float */ 
	sprintf(filename,"image%dx%d.ppm",width,height); /* create filename */
	sprintf(command,"xv %s",filename); /* create command to display   */
	
	setbuf(stdout,NULL); /* dont buffer terminaloutput */
	printf("\nfractal image in TCC /enthusi\n\n");
	printf("start at:  ");
	system("date +%k:%M:%S.%N"); /* use 'date' executable */
	printf("opening files............");
	if(!(dpic=fopen(filename,"w")))
		{fprintf(stderr,"file-error\n");return -1;}
	else printf("done\n");

	printf("writing head of file.....");
	fprintf(dpic,"P6\n%d %d\n255\n",width,height);
	printf("done\n");

	printf("mapfill in progress.......c64");

	for (yp=0;yp<height;yp++)
		{
  		ib=atof(argv[1]); /* reset start of x-parameter */
		for (xp=0;xp<width;xp++)
			{
			ix=0;
			iy=0;
			loop=0;

			do	{
				xs=ix*ix;			/* this is the		*/
				ys=iy*iy;			/* actual iteration */
				nix=xs-ys+ib;		
				iy=2*ix*iy+ia;		/* new iy for next round */
				ix=nix;				/* new ix for next round */
				loop++;
				}	while((loop<maxloop)&&(xs+ys<4));
						
			/* now just some nice colorscheme			*/
			/* this will give full red twice between	*/
			/* loop=0 and loop=maxloop and so on		*/

			pixel.red=((float)((loop*512.0)/maxloop));
			pixel.green=((float)((loop*1024.0)/maxloop));
			pixel.blue=((float)((loop*2048.0)/maxloop));
			ib+=steprange; /* step in x */
		 	fwrite(&pixel,3,1,dpic);
			}
		ia+=steprange; /* step in y */
		printf("\b\b\b%2d%%",(int)(yp*100/height)); /* progress */
		}
		printf("\b\b\b\bdone\n");

	CHECK	/* use our defined printf()-check in line 140! */
	
	printf("writing data to file.....");
	fclose(dpic);	/* close and flush file */

	printf("done\n");
	printf("ending at: ");
	system("date +%k:%M:%S.%N"); /* against display exact time */
	printf("filename is: %s\n",filename);
	system(command); /* execute command-line to display image  */

	printf("\n");
	return 0;	/* exit program without errors */
	}
